#! /usr/bin/perl -w

# Copyright 2002-2004 by Hans Reiser, licensing governed by reiserfsprogs/README

# A mongo log files parser and output files (currently .html)
# producer. 

# print a proper usage pattern
sub USAGE
{
    print "usage: mongo_parser.pl <mongo log file #1> [<mongo log file #2> ...[<mongo log file #n>]...] ";
    exit -2;
}

# a variable needed by &get_cmd
$CMD_G = "";

# reset file reader;
sub reset_cmd 
{
    $CMD_G = "";
}

# get one line with a specific keyword at the beginning
sub get_cmd ($$) 
{
    my ($file, $keyword) = @_;

    unless ($CMD_G) {
	$CMD_G = <$file>;

	return undef unless ($CMD_G);

	die ("Log file contains an ERROR message\n") if ($CMD_G =~ /^ERROR/);

	chomp $CMD_G;
    }

    return undef unless ($CMD_G =~ /^${keyword}/);

    my $cmd = $CMD_G;
    $CMD_G = "";

    return $cmd;

}

# get all lines with satisfy given pattern
sub get_all_cmds ($$)
{
    my ($file, $keyword) = @_;

    my %result = ();
    my $cmd;
    my $cmd_found = 0;

    while ($cmd = &get_cmd($file, $keyword)) {
	my ($not_used, $val) = split / +/, $cmd, 2;
	$cmd_found = 1;

	if (defined $val) {
	  my ($desc_name, $desc_val) = (($keyword =~ /INFO/ ) ? split / : /,$val,2
					: split /=/,$val,2);
	  $result{$desc_name} = $desc_val;
	}
    }

    return \%result if $cmd_found;

    return undef;
}

# get all log file records for one phase ("create", "copy", ...)
sub get_phase ($)
{
    my ($file) = @_;

    my $cmd;

    $cmd = &get_cmd($file, "PHASE");

    return undef unless ($cmd);

    my ($not_used, $name) = split / +/, $cmd, 2;
    
    my @stats;
    
    while ($cmd = &get_cmd($file, "STAT")) {
	my ($not_used, $stat_name, $stat_value) = split / +/, $cmd, 3;
	push @stats, {"name"=>$stat_name, "value"=>$stat_value};
    }

    my %result;

    $result{"name"} = $name;
    $result{"stats"} = \@stats;

    return \%result;
}


# get all records for one set of series from the log file and process
# them.
sub get_series ($)
{
    my ($file) = @_;

    my $desc;

    $desc = &get_all_cmds($file, "SER");

    return undef unless ($desc);

    my $desc_str = "";

    foreach my $key (keys %{$desc}) {
      $desc_str = $desc_str . $key . "=" . $desc->{$key} . " ";
    }

    my (@phases, $phase);

    push @phases, $phase while ($phase = &get_phase($file));

    my %result;
    
    $result{"desc"}   = $desc_str;
    $result{"phases"} = \@phases;

    return \%result;
}

# get a full test information from an input stream. it is a function
# which parses whole log file actually.
sub get_test ($)
{
    my ($file) = @_;
    my $desc;
    my $info;

    $info = &get_all_cmds($file, "INFO");
    die ("can\'t get a INFO description") unless ($info);

    $desc = &get_all_cmds($file, "TEST");

    die ("can\'t get a test description") unless ($desc);

    my @series;
    my $s;

    push @series, $s while ($s = &get_series($file));

    my %result;

    $result{"info"} = $info;
    $result{"desc"} = $desc;
    $result{"series"} = \@series;

    return \%result;
}

#
# parse a mongo log file and fill a hierarchical data structure
# 
sub parse_log ($)
{
    my ($log) = @_;

    open (LOG,  "<${log}") or die "can\'t open log file \"$log\": $!\n";

    my $test = &get_test(LOG);

    close LOG;

    return $test;
}

# the ratio between corresponded values is printed with different
# colors, one color for  0.98 <= ratio <= 1.02, and two colors for
# cases when ratio less than 0.98 and greater than 1.02

$GREATER = "red";
$LESS    = "green";
$EQUAL   = "black";

# get a color based on given ratio
sub get_color ($)
{
    my ($ratio) = @_;

    return $LESS  if $ratio < 0.98;
    return $EQUAL if $ratio <= 1.02; 
    return $GREATER;
}

# returns a A,B,C,... if you passed 0,1,2,... as a parameter
sub letter_by_number ($)
{
    my ($n) = @_;
    return chr(ord('A') + $n);
}

# return a highlighted string if value is equal to the specified "best value"
sub mark_best_value ($$$)
{
    my ($best, $val, $str_to_change) = @_;

    return $str_to_change unless defined ($best) && defined ($val);

    return $str_to_change unless ($val <= $best * 1.02);

    return "<U> $str_to_change</U>";
}

sub detect_not_common_set ( @ )
  {
    my @sets;

    foreach my $tree (@_) {
      push @sets, $tree->{"desc"};
    }

    my %not_common_set;
    my %all_opt_names;

    foreach my $href (@sets) {
      foreach my $name (keys %$href) {
	my $val = $href->{$name};
	$all_opt_names{$name} = $val;
      }
    }

    foreach my $href (@sets) {
      foreach my $name (keys %all_opt_names) {
	my $val = $href->{$name};
	my $old_val = $all_opt_names{$name};

	$not_common_set{$name} = 1 if (! defined ($val) || $val ne $old_val);
      }
    }

    return %not_common_set;
  }

#to sort in revers
sub backwards {
    $b cmp $a;
}
#
# process the data structures and produce an HTML output
#
sub compare (@)
{
    my @trees = @_;

    print "<html><style>\nDT{font-weight: bold}\n</style>\n<body text=black>\n";

    my %not_common_opts = detect_not_common_set(@trees);

    my $n_trees = $#trees + 1;

    # Traverse a hierarchical structure referenced by the first
    # parameter (first element of arguments array) and find
    # corresponding elements in a structure referenced by the second
    # parameter. It constructs and prints HTML as well.

    # first tree is a reference data structure
    my $a_tree = $trees[0];

    my $n_series = $#{$a_tree->{"series"}} + 1;
    my $n_stats  = $#{$a_tree->{"series"}[0]->{"phases"}[0]->{"stats"}} + 1;

    my @stat_names;

    for (my $i = 0; $i < $n_stats; $i ++) {
	push @stat_names, $a_tree->{"series"}[0]->{"phases"}[0]->{"stats"}[$i]->{"name"};
    }

    my $cols = $n_stats * $n_trees + 1;

    # print benchmark INFO table header
    print "<dl>";

    my @foo_keys = sort backwards keys %{$trees[0]->{"info"}};
    foreach my $key (@foo_keys) {
      next if defined $not_common_opts{$key};
      print "\n<dt>";
      print $key;
      print "</dt>\n<dd>";
      print (($trees[0]->{"info"})->{$key}) if (($trees[0]->{"info"})->{$key}) ;
      print "</dd>";
    }
    print "\n</dl>\n";

    print "<table cols=$cols cellpadding=2 cellspacing=2 noborder>\n";
    print "<tr><td bgcolor=black colspan=$cols><font color=white></td></tr>\n";

    for (my $i = 0; $i < $n_trees; $i ++) {

	my $letter = &letter_by_number($i);
	my $desc = $trees[$i]->{"desc"};

	print "<tr>\n";
	print "<th bgcolor=#303030 colspan=$cols align=left><font color=white>${letter}.";

	foreach my $key (keys %$desc) {
	  next unless defined $not_common_opts{$key};
	  print $key . "=" . $desc->{$key} . " ";
	}

	print "</font></th>\n";
	print "</tr>\n";
    }

    # print series
    for (my $ser_x = 0; $ser_x < $n_series ; $ser_x ++) {
	my $n_phases ;
	
#	for my $tree (@trees) {
#	    my $ser = $tree->{"series"}[$ser_x];
#	    my $desc = $ser->{"desc"};

#	print "<tr> <td colspan=$cols bgcolor=white></td></tr>\n";

	my $a_desc = $a_tree->{"series"}[$ser_x]->{"desc"};
	my $view_head = $a_desc; # it will be the key for the hash we will pass to the bar-chart routine

	print "<tr> <td colspan=$cols bgcolor=#606060><b><font color=white>#${ser_x}:$a_desc</font></b></td></tr>\n";
#	}

	# print a row with stat names, first cell is empty.
	print "<tr align=center bgcolor=#C0C0C0> <td></td>\n";

	my $colspan = $n_trees;

	foreach my $st (@stat_names) {
	    print "<td colspan=${colspan}><b>$st</b></td>\n";
	}
	print "</tr>\n";

	# print a row with names of different input sources, first cell is empty.
	print "<tr align=center bgcolor=#C0C0C0> <td></td>\n";

	# column for a reference value from first data source (tree)
	my $all_source_names = "<td><b>A</b></td>";

	for (my $i = 1; $i < $n_trees; $i++) {
	    my $letter = letter_by_number ($i);
	    $all_source_names .= "<td><b>${letter}/A </b></td>";
	}

	print "$all_source_names\n" x $n_stats;
	print "</tr>\n";

	$n_phases = $#{$a_tree->{"series"}[$ser_x]->{"phases"}} + 1;

	# print test phases ("create", "copy", ..., "delete") in rows
	for (my $phase_x = 0; $phase_x < $n_phases; $phase_x ++) {

	    my $phase_name = $a_tree->{"series"}[$ser_x]->{"phases"}[$phase_x]->{"name"};
	    my $view_phase_head = $view_head . "  " . $phase_name;

	    print "<tr>\n";
	    print "<td bgcolor=#C0C0C0><b>${phase_name}</b></td>\n";

	    for (my $stat_x = 0; $stat_x < $n_stats; $stat_x++) {

	        #append a stat name to the view header
	              my $view_stat_head = $view_phase_head . ". " . 
		      $a_tree->{"series"}[$ser_x]->{"phases"}[$phase_x]->{"stats"}[$stat_x]->{"name"} . ":";

		# find a minimum (best) value.
		my $best_val;

		for (my $i = 0; $i < $n_trees; $i++) {
		    my $stat_val = $trees[$i]->{"series"}[$ser_x]->{"phases"}[$phase_x]->{"stats"}[$stat_x]->{"value"};

		    next unless defined  $stat_val;

		    $best_val = $stat_val if (!defined ($best_val) || ( $best_val > $stat_val));
		}

		# print a reference value 
		my $ref_stat_val = $trees[0]->{"series"}[$ser_x]->{"phases"}[$phase_x]->{"stats"}[$stat_x]->{"value"};
		my $out_val = $ref_stat_val;
		
		my %view_stat_by_letter ;
		$view_stat_by_letter{"A"} = $ref_stat_val;
		      unless (defined $ref_stat_val) {
			print STDERR "undefined reference value\n";
			exit -2;
		      }

		$out_val = mark_best_value ($best_val, $out_val, $out_val);

		print "<td bgcolor=#E0E0C0 align=right><tt>${out_val}</tt></td>\n";

		# print ratios with corresponded values from other data sources
		for (my $i = 1; $i < $n_trees; $i++) {
		    my $stat_val = $trees[$i]->{"series"}[$ser_x]->{"phases"}[$phase_x]->{"stats"}[$stat_x]->{"value"};
		    $view_stat_by_letter{&letter_by_number($i)} = $stat_val;
		    $out_val = $stat_val;

		    $out_val = '-' unless defined $stat_val;

		    if (!defined $stat_val) {
			print "<td bgcolor=#E0E0C0 align=right><tt> - </tt></td>\n";
		    } elsif (!defined ($ref_stat_val) or $ref_stat_val eq 0) {
			# print a fraction:
			print "<td bgcolor=#E0E0C0 align=right><tt>${stat_val}/${ref_stat_val}</tt></td>\n";
		    } else {
			my $ratio = $stat_val / $ref_stat_val;
			my $color = get_color($ratio);

			$out_val = sprintf "%10.3f", $ratio;

			$out_val = mark_best_value ($best_val, $stat_val, $out_val);

			print "<td bgcolor=#E0E0C0 align=right><tt>";
			printf "<font color=$color> ${out_val} </font>", $ratio;
			print "</tt></td>\n";
		    }
		}
		$view_hash_by_head{$view_stat_head} = \%view_stat_by_letter;
		print "</tt></td>\n";
	    }

	    print "</tr>\n";
	}

#	print "<tr><td bgcolor=black colspan=$cols><font color=white></td></tr>\n";

    }

    print "<tr><td bgcolor=black colspan=$cols><font color=white></td></tr>\n";

    print "<tr><td colspan=$cols align=right> \n";

    print "<tr> <td colspan=$cols bgcolor=#303030><b><font color=white>";

    foreach my $key (keys %{$trees[0]->{"desc"}}) {
      next if defined $not_common_opts{$key};
      print $key . "=" . ($trees[0]->{"desc"})->{$key} . " ";
    }
    print "</td></tr>\n";

    print "<tr><td colspan=$cols align=right> <font size=-2>Produced by <a href=http://namesys.com/mongo>Mongo</a> benchmark suite.</font></td></tr>\n";

    print "</table>\n";

    print "<!--
<p><b>Legend:</b> <font color=\"green\">green</font> color means the result is
better (less) than reference value from the first column, results
marked as <font color=\"red\">red</font> are worse than reference value,
best results are <u>underlined</u> other results which fit into 2%
margin of the best result are underlined also.</p>
-->";
    if ($create_bar_chart) {
	print "<p><a href=\"charts/comp.html\">The same results in the charts</a></p>";
    }
    print "</body></html>\n";
}

sub view($) {
   my ($ref) = @_;
   my %hash = %$ref;
   my $height = 20;
   local ($max, @keys, @norm, $value, %data, $colspan);
   %data = ();
   $max = 1;
   @keys = sort keys %hash;
   @norm = ();
   foreach $k (@keys){
          $value = $hash{$k};
      $max = $value if ($value > $max) ;
   }
   $keys = scalar @keys;
   foreach $k (@keys){
      $value = $hash{$k};
      $data{$k} = sprintf ("%0.0f", ($value * $height / $max + 0.5));
      $data{$k} = 0 if (!$value);      
   }

   $colspan = scalar @keys * 2 + 1;
   print "<center><table bgcolor=#000000 cellpadding=\"1\" cellspacing=\"0\" border=\"0\">\n<tr><td>\n";
   print "<table bgcolor=#ffffff cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n";
   print "<tr><td colspan=$colspan><img src=s.gif></td></tr>\n";
   for (my $h = $height; $h > 0; $h--){
      print "<tr><td><img src=s.gif></td>";
      foreach $v (keys %data){
         $p = ($data{$v} >= $h) ? "<td><img src=b.gif alt=\"$hash{$v}\"></td>" : "<td><img src=w.gif></td>";       
         print "$p<td><img src=s.gif></td>";
      }
      print "</tr>\n";
   }
   print "<tr><td><img src=s.gif></td>";
   foreach $k (@keys){
      print "<td align=center><font face=sans-serif size=0>$k</font></td><td><img src=s.gif></td>";
   }
   print "</tr>\n</table>\n</td></tr></table></center>\n";
}




# calculate range and frequency by max value 
sub range_freq {
    my $max = shift;
    my $freq = 50;

    my $range = (($max - ($max % $freq)) / $freq + 1 ) * $freq;
    return ($range, $freq)
}

# each filesystem is marked its own color (line style param) 
sub what_ls {
    my $fs = shift;
    
    return 1 if ($fs =~ /ext2/);
    return 2 if ($fs =~ /ext3/);
    return 3 if ($fs =~ /notail/);
    return 4 if ($fs =~ /reiser4/);
    return 8 if ($fs =~ /reiserfs/);
    return 10;
}

sub make_separate_plots_boxes {
    my $n_series = shift;
    my $file_names = shift;
    my $titles = shift;
    my $phtls = shift;
    my $n_phases = $#$phtls;
    my $tmp_cmd = "boxes.tmp.cmd";
#    my $png = "\nset output \"png.output.";
    my $foo_name = "eps.output.foo";
    my $png = "\nset output \"" . $foo_name;
    my $plot = "\nplot ";
    my $plottl = "\nset title \"Mongo Benchmark results (the low values are better). Phase  ";
    my $str;

    my @charts_html = (
		       "<html>",
		       "<body>"
		    );

    my @gnucmds_boxes = (
			 "set terminal postscript color\n",
			 "set linestyle 1 lt 1 lw 3 pt 1 ps 1.5 ;\n",
			 "set linestyle 2 lt 2 lw 3 pt 1 ps 1.5 ;\n",
			 "set linestyle 3 lt 3 lw 3 pt 1 ps 1.5 ;\n",
			 "set linestyle 4 lt 4 lw 3 pt 1 ps 1.5 ;\n",
			 "set linestyle 5 lt 5 lw 3 pt 1 ps 1.5 ;\n",
			 "set linestyle 6 lt 6 lw 3 pt 1 ps 1.5 ;\n",
			 "set linestyle 7 lt 7 lw 3 pt 1 ps 1.5 ;\n",
			 "set linestyle 8 lt 8 lw 3 pt 1 ps 1.5 ;\n",
			 "#set term png\n",
			 "set yrange [0:]\n",
			 "set grid\n",
			 "set xrange [0:2]\n",
			 "set noxtics\n", 
			 "set data style boxes\n",
			 "set boxwidth 0.2\n",
		   );


   # gnu BOXES
   for ($j = 0 ; $j <= $n_series; $j ++) {
       my @phases;
       open (F, "<${file_names}.${j}") or die "can\'t open tmp cmds file \"${file_names}.${j}\": $!\n";
       @phases = <F>;
       close F;

       my $k;
       for ($k = 0; $k <= $n_phases; $k ++) {
	   my $var = "vlb_" . $k;
	   $$var [$j] = $phases [$k];
	   chomp $$var [$j];
       }
   }

   # store to phase.file.j
    print $n_phases;
    my $k;
    for ($k = 0; $k <= $n_phases; $k ++) {
	my $file = "phase." . $k;
	open (FF, ">${file}") or die "can\'t open tmp cmds file \"$file\": $!\n";
	my $var = "vlb_" . $k;
	$str = "0 " . (join " ", @$var);
	print FF $str;
	close FF;
    }

    # set command for gnuplot
	print $#$titles ; 
    
    for ($j = 0; $j <= $#$titles; $j ++ ) {
	print $$titles [$j]; 
	# SKIP reiserfs rw
#	next if (($$titles [$j] =~ /reiserfs/)) and ($$titles [$j] =~ /rw/);
	my $ls = &what_ls ($$titles [$j]);

	$tmp_using [$j] = "using (\$1 + " . (($j + 1) * 0.2) . " ):( \$" . ($j + 2) . " ) " . $$titles [$j] . " ls " . $ls  ;
    }

    for ($i = 0; $i <= $n_phases; $i ++ ) {
	my $file = "\"phase." . $i . "\"";
	$gnucmds_boxes [++$#gnucmds_boxes] = $png . $i . "\"";
	$gnucmds_boxes [++$#gnucmds_boxes] = $plottl . $$phtls [$i] . "\""; 
	$gnucmds_boxes [++$#gnucmds_boxes] = $plot . $file . " " . (join "\, \"\" ", @tmp_using);
    }

    open (CMD, ">${tmp_cmd}") or die "can\'t open tmp cmds file \"$tmp_cmd\": $!\n";
    print CMD @gnucmds_boxes;
    close CMD;

    # run gnuplot
    my $gnurun = "cat ${tmp_cmd} | gnuplot ";
    `${gnurun}`;
     if ($?) {
	 die "can't run gnuplot, is it installed? : $!\n";
     }
    
    # run convert to filled boxes
    my $cmd = "/home/grev/gnuplot/boxfill-0.2/boxfill.pl "; 
    foreach ($i = 0; $i <= $n_phases; $i ++) {
	my $file = $foo_name . $i;
	my $filledfile = $$phtls [$i] ;
	my $make_and_convert = $cmd . $file .  " " . $filledfile . " \&\& convert -geometry 75x75% -rotate 90 " . $filledfile . " png:" . $filledfile . ".png";
	
	`${make_and_convert}`;
	if ($?) {
	    die "can't run convert! boxfill-02 (Andreas Widmann, widmann\@rz.uni-leipzig.de) and convert tool (ImageMagick 5.4.2 http://www.imagemagick.org ) should be installed : $!\n";
	}
	$charts_html [++$#charts_html] = "\n<img src=\"" . $filledfile . ".png\">";
	# remove temporary file 
	`rm $filledfile`;
    }

    # Make html
    $charts_html [++$#charts_html] = "\n\n</body>\n</html>\n";

    my $file = "charts_for_phases.html";
    open (F, ">$file") or die "can\'t open html file \"$file\": $!\n";
    print F @charts_html;
    close;


    # remove all temporary files
    `rm $tmp_cmd` if (-e $tmp_cmd);
    for ($i = 0; $i <= $n_phases; $i ++) {
	my $tmp = $foo_name . $i;
	`rm $tmp`;
	$tmp = "phase." . $i;
	`rm $tmp`;
    }
}

# make one tree plot by gnuplot (all SERies are included)
# temporary results with data for gnuplot are stored in files
sub make_plots {
    my @trees = @_;
    my %not_common_opts = detect_not_common_set(@trees);

    my $plt_cmd = "tmp.gnu.plt";
    my $tmp_data = "tmp.gnu.data";
    my $set_out = "\nset output \'tmp.";
    my $set_outend = "\.eps'";
    my $set_title = "\nset title \"Mongo comparison : phase ";
#    my $set_titleend = " \" -10, 0";
    my $set_titleend = " \" ";
    my $size = "";
    my @titles;
    my @gnucmds = (
		   # see gnuplot help : eps reduce the size of plot to
		   # half of the default size 
		   "set terminal postscript eps color \"Helvetica\" 24",
		   "set border 3",
		   "set xlabel \"the low values are better\"",
		   "set xrange [0:4]",
		   "set yrange [0:]",
		   #"set title \"Mongo comparison (the low values are better)\" -20, 0",
		   "set xtics nomirror ",
		   "set ytics nomirror ",
		   "set noxtics\n", 
		   "set grid",
		   "set data style boxes",
		   "set boxwidth 0.2");
		   #"plot ");
    # All trees 
    for (my $k = 0; $k <= $#trees; $k ++ ) {
	my $tree = $trees[$k];

	my @vl;
	my $n_phases = $#{$tree->{"series"}[0]->{"phases"}};
	my $n_series = $#{$tree->{"series"}};
	for (my $j = 0; $j <= $n_series; $j ++) {
	    for (my $i = 0; $i <= $n_phases; $i ++ ) {
		my $phase = $tree->{"series"}[$j]->{"phases"}[$i]->{"name"};
		# REAL_TIME -- stats [0];
		$vvalue  = $tree->{"series"}[$j]->{"phases"}[$i]->{"stats"}[0]->{"value"};

		# just to rewrite old file
		if ($k == 0 ) {
		    open (D, ">${tmp_data}.${phase}") or die "can\'t open tmp data file \"$tmp_data\.$phase\": $!\n";
		    print D  "0.4 " . $vvalue . " ";
		} else {
		    open (D, ">>${tmp_data}.${phase}") or die "can\'t open tmp data file \"$tmp_data\.$phase\": $!\n";
		    print D  $vvalue . " ";

		}
		close D;
	    }

	    $title = "";
	    my $ddsc = $tree->{"desc"};
	    foreach my $key (keys %$ddsc) {
		if ($key =~ "FILE_SIZE") {
		    $size = $ddsc->{$key};
		}
		next unless defined $not_common_opts{$key};
		my $vl = $ddsc->{$key}; 
		$vl =~ s/\"/\\\"/;
		# This works just for \" on the end of string
		$vl =~ s/\"$/\\\"/;
		
		# key and its value will be writen as the title of every "box"
		# but this title is very long for small plots and it is better
		# to exclude some keys (MOUNT_OPTIONS for example) and write
		# only value.
		# $title = $title . " " . $key . "=" . $vl;
		if ($key =~ "MOUNT_OPTIONS" ) {
		    $title = $title . " " . $vl;
		} elsif (($key =~  /MKFS/) && ($vl =~ /(.*)policy\=(.*)/) ) {
		    $title = $title . " " . $2;
		} elsif ($key =~  "MKFS") {
			$title = $title;
		} else {
		    $title = $title . " " . $key . "=" . $vl; 
		}
	    }
	    $title = "title \"" . $title . "\"";
	    
	    $titles [++$#titles] = $title; 
	}

	# Supposed that all serias have the same n_phases value and the same
	# set of phases
	for ($i = 0 ; $i <= $n_phases ; $i ++) {
	    $phases [++$#phases] = ($tree->{"series"}[0]->{"phases"}[$i]->{"name"});
	}
	foreach (@phases) {
	    next unless /Stats/;
	    @_ = "Stats";
	}
	
	# run gnuplot for making separate plots boxes for each phase    
	#&make_separate_plots_boxes ($n_series, $tmp_data, \@titles, \@phases) if ($create_phases_plots);
    
    }

    foreach (@gnucmds) {
	$_ = $_ . "\n";  
    }

    # create plots for all phases
    # it is supposed that the number of phases are the same for all trees 
    my $tree = $trees[0];
    my $n_phases = $#{$tree->{"series"}[0]->{"phases"}};
    for (my $i = 0; $i <= $n_phases; $i ++) {
	my $phase = $tree->{"series"}[0]->{"phases"}[$i]->{"name"};
	$gnucmds [++$#gnucmds] = $set_out . ${phase} . $set_outend . "\n";
	$gnucmds [++$#gnucmds] = $set_title . ${phase} . " file_size = " . $size . " " .$set_titleend . "\n";

	$gnucmds [++$#gnucmds] = "plot " . " \'${tmp_data}.${phase}\' " ;
	for ($t = 0 ; $t <= $#titles ; $t ++) {
	    my $end = ($t eq $#titles) ? ";" : ", \\\n"; 
	    my $empty = ($t eq 0 ) ? "" : " \'\' ";
	    $tmp_using [$t] = $empty . "using (\$1 + " . (($t + 1) * 0.2) . " ):( \$" . ($t + 2) . " ) " . $titles [$t]  ;
	    
	    $gnucmds [$#gnucmds] = $gnucmds [$#gnucmds] . $tmp_using [$t] . $end;
	}
    }
    open (CMD, ">${plt_cmd}") or die "can\'t open tmp cmds file \"$plt_cmd\": $!\n";
    print CMD @gnucmds;
    close CMD;


    # run gnuplot
     #my $gnurun = "cat ${plt_cmd} | gnuplot ";
    my $gnurun = "gnuplot  " . ${plt_cmd};
     `${gnurun}`;
    if ($?) {
        die "can't run gnuplot, is it installed? : $!\n";
    }

    # convert to filled boxes
    my $cmd = "/mnt/gnuplot/boxfill-0.2/boxfill.pl "; 
    for (my $i = 0; $i <= $n_phases; $i ++) {
	my $phase = $tree->{"series"}[0]->{"phases"}[$i]->{"name"};
	my $file = ${phase} . ".eps";
	my $fill_it = $cmd .  "tmp.${phase}.eps " . ${file};
	`${fill_it}`;
    }
    # convert it to png

    $cmd = "convert  -contrast  -contrast -contrast ";
    for (my $i = 0; $i <= $n_phases; $i ++) {
	my $phase = $tree->{"series"}[0]->{"phases"}[$i]->{"name"};
	my $file = ${phase} . ".eps";
	$cmd = "convert  -contrast  -contrast -contrast " . $file . " png:" . ${phase}. ".png" ;
	`${cmd}`;
    }

    `rm *.eps`;
    die " $!\n" if ($?);

    `rm tmp*`;
    die " $!\n" if ($?);
    `mkdir charts`;
    die " $!\n" if ($?);
    `mv *.png charts/.`;
    die " $!\n" if ($?);
   

}

sub print_bar_chart($){
  my ($href) = @_;
  foreach my $k (sort keys %$href){
    print "$k<br>\n";
    view($href->{$k});
  }
}

#
# MAIN
#

&USAGE if ($#ARGV == -1);

my @trees;
$create_bar_chart = 0;
$create_phases_plots = 0;
%view_hash_by_head=();

OUT: foreach my $arg (@ARGV) {
      if ($arg eq "--bar-chart") {
	$create_bar_chart = 1;
	next OUT;
      }
      if ($arg eq "--phases-plots") {
	$create_phases_plots = 1;
	next OUT;
      }
      &reset_cmd;
      my $parsed_tree = &parse_log($arg);
      push @trees, $parsed_tree;
    }
&compare (@trees);
&make_plots (@trees) if ($create_bar_chart);
#&print_bar_chart(\%view_hash_by_head) if ($create_bar_chart);

exit 0;










